==> wm-react-native-components-0.0.81.tgz: package/slidemenu/LeftSlideCellMenuView.js <==
/**
 * 左滑
 * Created by aaronlin on 2017/8/16.
 */
import React, { PureComponent } from 'react';
import PropTypes from 'prop-types';
import {
    Animated,
    PanResponder,
} from 'react-native';

import WMView from '../WMView';

/**
 * 最小的移动距离
 * @type {number}
 */
const MIN_MOVE_DISTANCE = 0.66;

/**
 * 左滑菜单 必须设置 style 中的 width
 *
 */
export default class LeftSlideCellMenuView extends PureComponent{

    /**
     *
     * @type {{deleget: *, rightMenuView}}
     */
    static propTypes = {
        deleget:PropTypes.object.isRequired,    //当前的Component
        rightMenuView:PropTypes.element,  //右边菜单 View
    }

    /**
     *
     * @type {null}
     * @private
     */
    _panResponder = null;
    /**
     * 右边菜单总宽度
     * @type {number}
     * @private
     */
    _menuWidth = 100;
    /**
     * childView 初始的 translateX
     * @type {number}
     * @private
     */
    _initChildTranslateX = 0;
    /**
     * 动画时间
     * @type {number}
     * @private
     */
    _duration = 180;

    /**
     *
     * @param props
     */
    constructor(props){
        super(props);

        let { rightMenuView, deleget } = this.props;
        if(rightMenuView && rightMenuView.props && rightMenuView.props.style && rightMenuView.props.style.width>0){
            this._menuWidth = rightMenuView.props.style.width;
        }

        if(this._menuWidth <= 0){
            throw new Error('请设置左滑菜单具体宽度!!');
        }

        if(!deleget){
            throw new Error('请传入当前的Component!!');
        }

        this.state = {
            childTranslateX: new Animated.Value(this._initChildTranslateX),
            menuTranslateX:new Animated.Value(this._menuWidth),
        }

        this._panResponder = PanResponder.create({
            // 要求成为响应者：
            onStartShouldSetPanResponder: (evt, gestureState) => false,
            onStartShouldSetPanResponderCapture: (evt, gestureState) => false,
            onMoveShouldSetPanResponder: (evt, gestureState) => {
                // console.log('onMoveShouldSetPanResponder===========> ', gestureState.dx);
                let { dx } = gestureState;
                return Math.abs(dx)>=MIN_MOVE_DISTANCE;
            },
            onMoveShouldSetPanResponderCapture: (evt, gestureState) => {
                // console.log('onMoveShouldSetPanResponderCapture===========> ', gestureState.dx);
                let { dx } = gestureState;
                return Math.abs(dx)>=MIN_MOVE_DISTANCE;
            },
            // 开始手势操作。给用户一些视觉反馈，让他们知道发生了什么事情！
            // gestureState.{x,y} 现在会被设置为0
            onPanResponderGrant: (evt, gestureState) => {

            },
            // 最近一次的移动距离为gestureState.move{X,Y}
            // 从成为响应者开始时的累计手势移动距离为gestureState.d{x,y}
            onPanResponderMove: (evt, gestureState) => {
                // console.log('onPanResponderMove=======> ', gestureState);

                let { dx } = gestureState;
                let menuTranslateX = this._menuWidth + dx;
                let childTranslateX = this._initChildTranslateX + dx;

                if(menuTranslateX<=0){
                    menuTranslateX = 0;
                    childTranslateX = -this._menuWidth;
                }else if(menuTranslateX>=this._menuWidth){
                    menuTranslateX = this._menuWidth;
                    childTranslateX = 0;
                }

                this._cacheCurrOpenMenu(true);

                this.state.childTranslateX.setValue(childTranslateX);
                this.state.menuTranslateX.setValue(menuTranslateX);
            },
            //监视器被要求终止，返回false表示不同意终止，或者不处理
            onPanResponderTerminationRequest: (evt, gestureState) => {
                this._autoOpenOrCloseMenu(false);
                return true;
            },
            // 用户放开了所有的触摸点，且此时视图已经成为了响应者。
            // 一般来说这意味着一个手势操作已经成功完成。
            onPanResponderRelease: (evt, gestureState) => {
                // console.log('onPanResponderRelease================>');

                this._autoOpenOrCloseMenu();
            },
            // 另一个组件已经成为了新的响应者，所以当前手势将被取消。
            onPanResponderTerminate: (evt, gestureState) => {
                this._autoOpenOrCloseMenu(false);
            },
            // 返回一个布尔值，决定当前组件是否应该阻止原生组件成为JS响应者
            // 默认返回true。目前暂时只支持android。
            onShouldBlockNativeResponder: (evt, gestureState) => {
                return true;
            },
        });
    }

    componentDidUpdate(nextProps, nextState){
        this.reset();
    }

    componentWillUnmount() {
        this.stopAnim();
    }

    /**
     * 菜单是否已经打开
     */
    isOpen = ()=>{
        return this.state.menuTranslateX._value === 0;
    }

    /**
     * 缓存当前打开的 菜单 对象
     * @private
     */
    _cacheCurrOpenMenu = (isOpen)=>{
        if(isOpen === undefined){
            isOpen = this.isOpen();
        }
        let currOpenMenu = this._getCurrOpenMenu();
        if(isOpen){  //当前已经打开
            if(currOpenMenu !== this){
                currOpenMenu && currOpenMenu.closeMenu();
                this._setCurrOpenMenu(this);
            }
        }else{  //当前关闭
            if(currOpenMenu === this){
                this._setCurrOpenMenu(null);
            }
        }
    }

    /**
     * 根据 menuTranslateX 判断 是否打开或关闭 menu
     * @private
     */
    _autoOpenOrCloseMenu = (isAnim = true)=>{
        if((this._menuWidth - this.state.menuTranslateX._value) >= this._menuWidth / 3){    //划过的 1/3 就默认打开菜单
            this.openMenu(isAnim)
        }else{
            this.closeMenu(isAnim);
        }
    }

    /**
     * 打开菜单
     * @param isAnim
     */
    openMenu = (isAnim = true)=>{
        this._cacheCurrOpenMenu(true);

        if(isAnim){
            Animated.parallel([
                Animated.timing(this.state.childTranslateX, {
                    fromValue: this.state.childTranslateX._value,
                    toValue: -this._menuWidth,
                    duration: this._duration,
                    isInteraction: false,
                }),
                Animated.timing(this.state.menuTranslateX, {
                    fromValue: this.state.menuTranslateX._value,
                    toValue: 0,
                    duration: this._duration,
                    isInteraction: false,
                }),
            ]).start();
        }else{
            this.state.childTranslateX.setValue(-this._menuWidth);
            this.state.menuTranslateX.setValue(0);
        }
    }

    /**
     * 关闭菜单
     * @param isAnim
     */
    closeMenu = (isAnim = true)=>{

        this._cacheCurrOpenMenu(false);

        if(isAnim){
            Animated.parallel([
                Animated.timing(this.state.childTranslateX, {
                    fromValue: this.state.childTranslateX._value,
                    toValue: 0,
                    duration: this._duration,
                    isInteraction: false,
                }),
                Animated.timing(this.state.menuTranslateX, {
                    fromValue: this.state.menuTranslateX._value,
                    toValue: this._menuWidth,
                    duration: this._duration,
                    isInteraction: false,
                }),
            ]).start();
        }else{
            this.state.childTranslateX.setValue(0);
            this.state.menuTranslateX.setValue(this._menuWidth);
        }
    }

    /**
     * 停止所有动画
     */
    stopAnim = ()=>{
        this.state.childTranslateX.stopAnimation();
        this.state.menuTranslateX.stopAnimation();
    }

    /**
     * 重置所有状态
     */
    reset = ()=>{
        this.stopAnim();
        this.closeMenu(false);
        this._closeCurrOpenMenu(false);
        this._setCurrOpenMenu(null);
    }

    /**
     * 获取当前打开的 菜单实例
     */
    _getCurrOpenMenu = ()=>{
        let { deleget } = this.props;
        if(deleget && deleget.currOpenMenu) {
            return deleget.currOpenMenu;
        }
    }

    /**
     * 设置当前打开的 菜单实例
     * @private
     */
    _setCurrOpenMenu = (openMenu) =>{
        let { deleget } = this.props;
        if(deleget) {
            deleget.currOpenMenu = openMenu;
        }
    }

    /**
     * 关闭当前打开的 菜单
     * @private
     */
    _closeCurrOpenMenu = (isAnim = true)=>{
        let currOpenMenu = this._getCurrOpenMenu();
        currOpenMenu && currOpenMenu.closeMenu(isAnim);
    }

    /**
     *
     * @param e
     * @private
     */
    _childOnStartShouldSetResponderCapture = (e)=>{
        let { rightMenuView } = this.props;
        if(!rightMenuView){
            return false;
        }
        if(this.isOpen()){
            this.closeMenu();
            return true;
        }else{
            this._closeCurrOpenMenu();
            this._setCurrOpenMenu(null);
        }
        return false;
    }

    /**
     * childView onLayout监听
     * @param nativeEvent
     * @private
     */
    _childLayout = (e)=>{
        let { layout } = e.nativeEvent;
        if(layout && layout.height>0){
            this.refs.menuView && (this.refs.menuView.setNativeProps({
                style:{height:layout.height}
            }))
        }
    }

    render(){
        let { rightMenuView } = this.props;

        let panHandlers = this._panResponder.panHandlers;
        if(!rightMenuView){
            panHandlers = null;
        }

        return (
            <WMView style={{flexDirection:'row', alignItems:'center', opacity:1}} {...panHandlers} pointerEvents={'box-none'}>

                <Animated.View pointerEvents={'box-none'} ref={'menuView'} style={[{position:'absolute', top:0, right:0, width:this._menuWidth, height:0, justifyContent:'center', alignItems:'center', opacity:1}, {transform:[{translateX:this.state.menuTranslateX}]}]}>
                    {rightMenuView}
                </Animated.View>

                <Animated.View
                    pointerEvents={'box-none'}
                    onLayout={this._childLayout}
                    style={[{flex:1, opacity:1}, {transform:[{translateX:this.state.childTranslateX}]}]}
                    onStartShouldSetResponder = {this._childOnStartShouldSetResponderCapture}
                    onStartShouldSetResponderCapture = {this._childOnStartShouldSetResponderCapture}
                >
                    {this.props.children}
                </Animated.View>

            </WMView>
        );
    }

}

==> wm-react-native-components-0.0.85.tgz: package/slidemenu/LeftSlideCellMenuView.js <==
/**
 * 左滑
 * Created by aaronlin on 2017/8/16.
 */
import React, { PureComponent } from 'react';
import PropTypes from 'prop-types';
import {
    Animated,
    PanResponder,
} from 'react-native';

import WMView from '../WMView';

/**
 * 最小的移动距离
 * @type {number}
 */
const MIN_MOVE_DISTANCE = 0.66;

/**
 * 左滑菜单 必须设置 style 中的 width
 *
 */
export default class LeftSlideCellMenuView extends PureComponent{

    /**
     *
     * @type {{deleget: *, rightMenuView}}
     */
    static propTypes = {
        deleget:PropTypes.object.isRequired,    //当前的Component
        rightMenuView:PropTypes.element,  //右边菜单 View
    }

    /**
     *
     * @type {null}
     * @private
     */
    _panResponder = null;
    /**
     * 右边菜单总宽度
     * @type {number}
     * @private
     */
    _menuWidth = 100;
    /**
     * childView 初始的 translateX
     * @type {number}
     * @private
     */
    _initChildTranslateX = 0;
    /**
     * 动画时间
     * @type {number}
     * @private
     */
    _duration = 180;

    /**
     *
     * @param props
     */
    constructor(props){
        super(props);

        let { rightMenuView, deleget } = this.props;
        if(rightMenuView && rightMenuView.props && rightMenuView.props.style && rightMenuView.props.style.width>0){
            this._menuWidth = rightMenuView.props.style.width;
        }

        if(this._menuWidth <= 0){
            throw new Error('请设置左滑菜单具体宽度!!');
        }

        if(!deleget){
            throw new Error('请传入当前的Component!!');
        }

        this.state = {
            childTranslateX: new Animated.Value(this._initChildTranslateX),
            menuTranslateX:new Animated.Value(this._menuWidth),
        }

        this._panResponder = PanResponder.create({
            // 要求成为响应者：
            onStartShouldSetPanResponder: (evt, gestureState) => false,
            onStartShouldSetPanResponderCapture: (evt, gestureState) => false,
            onMoveShouldSetPanResponder: (evt, gestureState) => {
                // console.log('onMoveShouldSetPanResponder===========> ', gestureState.dx);
                let { dx } = gestureState;
                return Math.abs(dx)>=MIN_MOVE_DISTANCE;
            },
            onMoveShouldSetPanResponderCapture: (evt, gestureState) => {
                // console.log('onMoveShouldSetPanResponderCapture===========> ', gestureState.dx);
                let { dx } = gestureState;
                return Math.abs(dx)>=MIN_MOVE_DISTANCE;
            },
            // 开始手势操作。给用户一些视觉反馈，让他们知道发生了什么事情！
            // gestureState.{x,y} 现在会被设置为0
            onPanResponderGrant: (evt, gestureState) => {

            },
            // 最近一次的移动距离为gestureState.move{X,Y}
            // 从成为响应者开始时的累计手势移动距离为gestureState.d{x,y}
            onPanResponderMove: (evt, gestureState) => {
                // console.log('onPanResponderMove=======> ', gestureState);

                let { dx } = gestureState;
                let menuTranslateX = this._menuWidth + dx;
                let childTranslateX = this._initChildTranslateX + dx;

                if(menuTranslateX<=0){
                    menuTranslateX = 0;
                    childTranslateX = -this._menuWidth;
                }else if(menuTranslateX>=this._menuWidth){
                    menuTranslateX = this._menuWidth;
                    childTranslateX = 0;
                }

                this._cacheCurrOpenMenu(true);

                this.state.childTranslateX.setValue(childTranslateX);
                this.state.menuTranslateX.setValue(menuTranslateX);
            },
            //监视器被要求终止，返回false表示不同意终止，或者不处理
            onPanResponderTerminationRequest: (evt, gestureState) => {
                this._autoOpenOrCloseMenu(false);
                return true;
            },
            // 用户放开了所有的触摸点，且此时视图已经成为了响应者。
            // 一般来说这意味着一个手势操作已经成功完成。
            onPanResponderRelease: (evt, gestureState) => {
                // console.log('onPanResponderRelease================>');

                this._autoOpenOrCloseMenu();
            },
            // 另一个组件已经成为了新的响应者，所以当前手势将被取消。
            onPanResponderTerminate: (evt, gestureState) => {
                this._autoOpenOrCloseMenu(false);
            },
            // 返回一个布尔值，决定当前组件是否应该阻止原生组件成为JS响应者
            // 默认返回true。目前暂时只支持android。
            onShouldBlockNativeResponder: (evt, gestureState) => {
                return true;
            },
        });
    }

    componentDidUpdate(nextProps, nextState){
        this.reset();
    }

    componentWillUnmount() {
        this.stopAnim();
    }

    /**
     * 菜单是否已经打开
     */
    isOpen = ()=>{
        return this.state.menuTranslateX._value === 0;
    }

    /**
     * 缓存当前打开的 菜单 对象
     * @private
     */
    _cacheCurrOpenMenu = (isOpen)=>{
        if(isOpen === undefined){
            isOpen = this.isOpen();
        }
        let currOpenMenu = this._getCurrOpenMenu();
        if(isOpen){  //当前已经打开
            if(currOpenMenu !== this){
                currOpenMenu && currOpenMenu.closeMenu();
                this._setCurrOpenMenu(this);
            }
        }else{  //当前关闭
            if(currOpenMenu === this){
                this._setCurrOpenMenu(null);
            }
        }
    }

    /**
     * 根据 menuTranslateX 判断 是否打开或关闭 menu
     * @private
     */
    _autoOpenOrCloseMenu = (isAnim = true)=>{
        if((this._menuWidth - this.state.menuTranslateX._value) >= this._menuWidth / 3){    //划过的 1/3 就默认打开菜单
            this.openMenu(isAnim)
        }else{
            this.closeMenu(isAnim);
        }
    }

    /**
     * 打开菜单
     * @param isAnim
     */
    openMenu = (isAnim = true)=>{
        this._cacheCurrOpenMenu(true);

        if(isAnim){
            Animated.parallel([
                Animated.timing(this.state.childTranslateX, {
                    fromValue: this.state.childTranslateX._value,
                    toValue: -this._menuWidth,
                    duration: this._duration,
                    isInteraction: false,
                }),
                Animated.timing(this.state.menuTranslateX, {
                    fromValue: this.state.menuTranslateX._value,
                    toValue: 0,
                    duration: this._duration,
                    isInteraction: false,
                }),
            ]).start();
        }else{
            this.state.childTranslateX.setValue(-this._menuWidth);
            this.state.menuTranslateX.setValue(0);
        }
    }

    /**
     * 关闭菜单
     * @param isAnim
     */
    closeMenu = (isAnim = true)=>{

        this._cacheCurrOpenMenu(false);

        if(isAnim){
            Animated.parallel([
                Animated.timing(this.state.childTranslateX, {
                    fromValue: this.state.childTranslateX._value,
                    toValue: 0,
                    duration: this._duration,
                    isInteraction: false,
                }),
                Animated.timing(this.state.menuTranslateX, {
                    fromValue: this.state.menuTranslateX._value,
                    toValue: this._menuWidth,
                    duration: this._duration,
                    isInteraction: false,
                }),
            ]).start();
        }else{
            this.state.childTranslateX.setValue(0);
            this.state.menuTranslateX.setValue(this._menuWidth);
        }
    }

    /**
     * 停止所有动画
     */
    stopAnim = ()=>{
        this.state.childTranslateX.stopAnimation();
        this.state.menuTranslateX.stopAnimation();
    }

    /**
     * 重置所有状态
     */
    reset = ()=>{
        this.stopAnim();
        this.closeMenu(false);
        this._closeCurrOpenMenu(false);
        this._setCurrOpenMenu(null);
    }

    /**
     * 获取当前打开的 菜单实例
     */
    _getCurrOpenMenu = ()=>{
        let { deleget } = this.props;
        if(deleget && deleget.currOpenMenu) {
            return deleget.currOpenMenu;
        }
    }

    /**
     * 设置当前打开的 菜单实例
     * @private
     */
    _setCurrOpenMenu = (openMenu) =>{
        let { deleget } = this.props;
        if(deleget) {
            deleget.currOpenMenu = openMenu;
        }
    }

    /**
     * 关闭当前打开的 菜单
     * @private
     */
    _closeCurrOpenMenu = (isAnim = true)=>{
        let currOpenMenu = this._getCurrOpenMenu();
        currOpenMenu && currOpenMenu.closeMenu(isAnim);
    }

    /**
     *
     * @param e
     * @private
     */
    _childOnStartShouldSetResponderCapture = (e)=>{
        let { rightMenuView } = this.props;
        if(!rightMenuView){
            return false;
        }
        if(this.isOpen()){
            this.closeMenu();
            return true;
        }else{
            this._closeCurrOpenMenu();
            this._setCurrOpenMenu(null);
        }
        return false;
    }

    /**
     * childView onLayout监听
     * @param nativeEvent
     * @private
     */
    _childLayout = (e)=>{
        let { layout } = e.nativeEvent;
        if(layout && layout.height>0){
            this.refs.menuView && (this.refs.menuView.setNativeProps({
                style:{height:layout.height}
            }))
        }
    }

    render(){
        let { rightMenuView } = this.props;

        let panHandlers = this._panResponder.panHandlers;
        if(!rightMenuView){
            panHandlers = null;
        }

        return (
            <WMView style={{flexDirection:'row', alignItems:'center', opacity:1, overflow:'hidden'}} {...panHandlers} pointerEvents={'box-none'}>

                <Animated.View pointerEvents={'box-none'} ref={'menuView'} style={[{position:'absolute', top:0, right:0, width:this._menuWidth, height:0, justifyContent:'center', alignItems:'center', opacity:1}, {transform:[{translateX:this.state.menuTranslateX}]}]}>
                    {rightMenuView}
                </Animated.View>

                <Animated.View
                    pointerEvents={'box-none'}
                    onLayout={this._childLayout}
                    style={[{flex:1, opacity:1}, {transform:[{translateX:this.state.childTranslateX}]}]}
                    onStartShouldSetResponder = {this._childOnStartShouldSetResponderCapture}
                    onStartShouldSetResponderCapture = {this._childOnStartShouldSetResponderCapture}
                >
                    {this.props.children}
                </Animated.View>

            </WMView>
        );
    }

}
